#include <analysis.h>
#include <utility.h>
#include <ansi_c.h>
#include <string.h>
#include <math.h>
#include <gpib.h>
#include <stdio.h>
#include <formatio.h>
#include <visa.h>
#include <userint.h>
#include <cvirte.h> /* Needed if linking in external compiler; harmless otherwise */

// Include R&S CMU driver header file
#include "rscmu200.h"
#include "rscmuk6w.h"


/*****************************************************************************/
/*  LOCAL CONSTANTS DEFINITION                                               */
/*****************************************************************************/

#define RSCMU_EXAMPLE_VERSION      "1.11"       // EXAMPLE version string
#define RSCMU_EXAMPLE_CREATIONDATE "09/2011"    // EXAMPLE creation month/year
#define RSCMU_EXAMPLE_CVIVERSION   "6.0"        // Used Labwindows/CVI-version

#define RSCMU_WCDMA_EXAMPLE_REVISION "Example Rev " RSCMU_EXAMPLE_VERSION", "RSCMU_EXAMPLE_CREATIONDATE", CVI "RSCMU_EXAMPLE_CVIVERSION // Instrument driver revision


// Variable declaration for secondary address for Base definition and for
// Function Groups definition

static  ViSession     g_hRsCmuWcdma_NSig = VI_NULL;
static  ViSession     g_hRsCmuWcdma_Sig  = VI_NULL;
static  ViSession     g_hRsCmuBase       = VI_NULL;

    // Macro "CheckCmuError" definition
#ifndef CheckCmuError
#define CheckCmuError(Code) if (_error = (DoCheckCmuError(__FILE__,__LINE__,Code)), (_error = (_error < 0) ? _error : 0)) \
  {goto Error;} else
#endif

// ... function definition for the previous macro
int DoCheckCmuError (char *File, int Line, ViStatus CmuStatus)
{
    ViChar  errorMessage[256];
    ViChar l_vErrBuf[512];
    ViInt32 _error;
    ViChar System_Error_Message[256];
    ViChar l_vCmuErrMsg[256];
    ViUInt16 stbByte = 0;
    ViStatus ErrorCode;
    ViChar l_vErrBuf2[256];

    if (CmuStatus != VI_SUCCESS)
    {
       do
       {
       RSCMU_ErrorQuery (g_hRsCmuBase, &ErrorCode, System_Error_Message);
       viReadSTB (g_hRsCmuBase, &stbByte);
       } while (stbByte==0x4);

        RSCMU_WCDMA_errorMessage (VI_NULL, CmuStatus, l_vCmuErrMsg);
        Fmt (l_vErrBuf, "%s<Error in file %s, Line %d: %s", File, Line, l_vCmuErrMsg);
        MessagePopup ("ERROR MESSAGE (CMU)", l_vErrBuf);

        Fmt (l_vErrBuf2, "Error Code:   %d, Error Message:   %s\n\n", ErrorCode, System_Error_Message);
        MessagePopup ("ERROR CODE (CMU)", l_vErrBuf);

        return -1;
    }
    return 0;

Error:
    return -1;
}

// Macro "WriteCmuViInt32Value" definition (Check if no result could be
// obtained (yet), or if the result obtained is invalid).
#ifndef WriteCmuViInt32Value
#define WriteCmuViInt32Value(String,ViInt32Value) \
(ViInt32Value != RSCMU_NAN_VI_INT32) ? printf ("%s%d\n", String, ViInt32Value) : printf ("%sNot a number\n", String);
#endif

// Macro "WriteCmuViReal64Value" definition (Check if no result could be
// obtained (yet), or if the result obtained is invalid).
#ifndef WriteCmuViReal64Value
#define WriteCmuViReal64Value(String,ViReal64Value) \
(ViReal64Value != RSCMU_NAN) ? printf ("%s%lf\n", String, ViReal64Value) : printf ("%sNot a number\n", String);
#endif




/*****************************************************************************/
/*======================== Main =============================================*/
/*****************************************************************************/

int main (int argc, char *argv[])
{

  ViChar     l_vInstrDriverRev[256];
  ViChar     l_vInstrFirmwareRev[256];

  ViInt32    _error;
  ViChar     errorMessage[256];

  int        ue_chan = 0;
  double     ue_freq = 0;
  int        bs_chan = 0;
  double     bs_freq = 0;
  int        outp_pow = -30;

  ViInt32    signalling_state = 0;
  ViInt32    registration = 0;
  ViInt32    paging = 0;
  ViInt32    connection = 0;
  ViInt32    registration_identity_type;
  ViChar     imei;
  ViChar     ue_dialed_number;
  ViChar     ue_calling_number;
  ViInt32    lossles_srns_relocation;
  ViInt32    rf_2507;
  ViInt32    header_compression;
  ViInt32    am_buffer_size;
  ViInt32    max_rlc_window_size;
  ViInt32    max_am_entities;
  ViInt32    sim_transport_ch_d;
  ViInt32    sim_cctrch_d;
  ViInt32    tti_trans_blocks_d;
  ViInt32    number_of_tfc_d;
  ViInt32    number_of_tf_d;
  ViInt32    turbo_decoding_d;
  ViInt32    rec_bits_all_d;
  ViInt32    rec_bits_conv_coded_d;
  ViInt32    rec_bits_turbo_coded_d;
  ViInt32    dpch_codes_d;
  ViInt32    physical_chn_bits_d;
  ViInt32    sf_512_d;
  ViInt32    sim_transport_ch_u;
  ViInt32    sim_cctrch_u;
  ViInt32    tti_trans_blocks_u;
  ViInt32    number_of_tfc_u;
  ViInt32    number_of_tf_u;
  ViInt32    turbo_decoding_u;
  ViInt32    rec_bits_all_u;
  ViInt32    rec_bits_conv_coded_u;
  ViInt32    rec_bits_turbo_coded_u;
  ViInt32    dpdch_bits_per_10ms;
  ViInt32    pcpch;
  ViInt32    ue_power_class;
  ViInt32    freq_separation;
  ViInt32    utra_fdd_tdd;
  ViInt32    gsm;
  ViInt32    multi_carr_mode;
  ViInt32    dl_compressed_mode;
  ViInt32    ul_compressed_mode;

  int        k = signalling_state;
  int        l = registration;
  int        m = paging;
  int        n = connection;

  ViInt32    measurement_status_maxpow = 0;
  ViInt32    counter_cycle_maxpow = 0;
  ViInt32    counter_evalperiod_maxpow = 0;

  ViReal64    upp_lim_peak;
  ViReal64    upp_lim_rms;
  ViReal64    low_lim_peak;
  ViReal64    low_lim_rms;
  ViReal64    rat_val_peak;
  ViReal64    rat_val_rms;

  ViReal64    Peak_Power_Current = 0;
  ViReal64    Peak_Power_Average = 0;
  ViReal64    Peak_Power_Maximum = 0;
  ViReal64    RMS_Power_Current = 0;
  ViReal64    RMS_Power_Average = 0;
  ViReal64    RMS_Power_Minimum = 0;
  ViReal64    RMS_Power_Maximum = 0;
  ViReal64    Out_Of_Tolerance = 0;

  ViInt32     measurement_status_wcdma = 0;
  ViInt32     counter_cycle_wcdma = 0;
  ViInt32     counter_evalperiod_wcdma = 0;

  ViReal64    evm_peak_wcdma = 0;
  ViReal64    evm_rms_wcdma = 0;
  ViReal64    magn_error_wcdma = 0;
  ViReal64    magn_error_rms_wcdma = 0;
  ViReal64    phase_error_peak_wcdma = 0;
  ViReal64    phase_error_rms_wcdma = 0;
  ViReal64    iq_origin_offset_wcdma = 0;
  ViReal64    iq_imbalance_wcdma = 0;
  ViReal64    freq_err_wcdma = 0;
  ViReal64    wav_quality_wcdma = 0;
  ViReal64    peak_cd_err_wcdma = 0;
  ViReal64    transmit_time_err_wcdma = 0;
  ViReal64    ue_pow_curr_wcdma = 0;
  ViReal64    out_of_tolerance_wcdma = 0;
  ViInt32     slot_nr_wcdma = 0;
  
  ViInt32     error_vector_magn_peak_wcdma_matc = 0;
  ViInt32     error_vector_magn_rms_wcdma_matc = 0;
  ViInt32     magn_err_peak_wcdma_matc = 0;
  ViInt32     magn_err_rms_wcdma_matc = 0;
  ViInt32     phase_err_peak_wcdma_matc = 0;
  ViInt32     phase_err_rms_wcdma_matc = 0;
  ViInt32     iq_origin_offset_wcdma_matc = 0;
  ViInt32     iq_imbalance_wcdma_matc = 0;
  ViInt32     freq_err_wcdma_matc = 0;
  ViInt32     wav_qual_wcdma_matc = 0;
  ViInt32     peak_cd_err_wcdma_matc = 0;
  ViInt32     transmit_time_err_wcdma_matc = 0;
  
  ViInt32     meas_stat_BER = 0;
  ViInt32     counter_stat_cycle_BER = 0;
  ViInt32     counter_eval_period_BER = 0;
  ViReal64    ber = 0.0;
  ViReal64    bler = 0.0;
  ViReal64    dbler = 0.0;
  ViInt32     dl_ul_alignment = 0;
  ViInt32     lost_transp_blocks = 0;
  ViInt32     BER_lim_match = 0;
  ViInt32     BLER_lim_match = 0;
  ViInt32     DBLER_lim_match = 0;
  
  float       ber_lim = 0;
  float       bler_lim = 0;
  float       dbler_lim = 0;
  float       ber_power = -50;
  float       step_size_ber = 0.2;
  float       ber_pow_lim = -100;
  float       bler_pow_lim = -100;
  float       dbler_pow_lim = -100;
  float       ber_power1 = 0;
  float       ber1 = 0;
  float       bler1 = 0;
  float       dbler1 = 0;
  float       dl_ul_alignment1 = 0;
  float       lost_transport_blocks1 = 0;
  float       ber_power2 = 0;
  float       ber2 = 0;
  float       bler2 = 0;
  float       dbler2 = 0;
  float       dl_ul_alignment2 = 0;
  float       lost_transport_blocks2 = 0;
  float       BER_lim_match1 = 0;
  float       BLER_lim_match1 = 0;
  float       DBLER_lim_match1 = 0;
  float       BER_lim_match2 = 0;
  float       BLER_lim_match2 = 0;
  float       DBLER_lim_match2 = 0;
               
  
  FILE *fp;
  
  char        a = 'h';
  char        b = 'z';
  char        c = 'r';  
  char        d = 's';
  char        e = 't';
  char        f = 'u';
  char        g = 'v';
  
  
if (InitCVIRTE (0, argv, 0) == 0) /* Needed if linking in external compiler; harmless otherwise */
   return -1; /* out of memory */
 
 Cls();
  
  // ==========================================================================
  // CMU BASIC UNIT
  // --------------------------------------------------------------------------
  // BASIC UNIT INITIALISATION
  //  -> GPIB board index is assigned to 0
  //  -> Primary address to the GPIB bus is assigned to the default (20)
  //  -> Ask for the indentifier of the CMU, reset the instrument
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_Init_Base (&g_hRsCmuBase, RSCMU_GPIB, 0, 20, RSCMU_DO_QUERY, RSCMU_RESET_ON, errorMessage));
  
  // --------------------------------------------------------------------------
  // FUNCTION GROUPS CONFIGURATION
  //   Subsystem Remote
  //     -> Secondary addresses of the GPIB bus are assigned for all available
  //        function groups
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_Base_SystRemAddrSec (g_hRsCmuBase, 2, RSCMU_WCDMA1900FDDMS_SIG, errorMessage)); 
  
  // --------------------------------------------------------------------------
  // FUNCTION GROUPS INITIALISATION
  // --------------------------------------------------------------------------
  //CheckCmuError (RSCMU_Init_RF_NSig (g_hRsCmuBase, &g_hRsCmuRf_NSig, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_Init (g_hRsCmuBase, &g_hRsCmuWcdma_Sig, RSCMU_WCDMA1900FDDMS_SIG, errorMessage));
  
  // --------------------------------------------------------------------------
  // -> Set a specified timeout value for all available function groups.
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_SetTimeoutAttribute (g_hRsCmuWcdma_Sig, 10000, errorMessage));
  
 
  CheckCmuError (RSCMU_WCDMA_RevisionQuery (g_hRsCmuBase, l_vInstrDriverRev, l_vInstrFirmwareRev, errorMessage));
  
  // --------------------------------------------------------------------------
  printf ("Instrument Driver Revision  (WCDMA) : %s\n", l_vInstrDriverRev);
  printf ("Instrument Firmware Revision      : %s\n", l_vInstrFirmwareRev);
  printf ("\n");
   
  // Opens the file "RsCmu.iee" for output IEEE bus commands.
//  CheckCmuError (RSCMU_Trace_ON (g_hRsCmuBase, "RsCmu.iee", errorMessage));
  
  // Select the Operating Band of the UE: Operating Band 1 (K68) 
  
  CheckCmuError (RSCMU_WCDMA_Sig_ConfOperatingBand (g_hRsCmuWcdma_Sig, 0, errorMessage));
  
  
   
  //------------------------
  //UE SIGNAL CONFIGURATION:
  //------------------------
  //    Set the channel number (frequency corresponding) of the RF signal analyzed 
  //    Define an offset of 0.0 kHz for the RF analyzer frequency
  //    Define the number of the long code that is used to scramble the uplink WCDMA signal as 105
  //    Define the slot number to be deduced from the pilot bits in the captured signal
  //    Define which DPCH contributions are used for the correlation analysis: DPCCH
  //    Define the minimum signal strength of each DPDCH in theWCDMA signal to be
  //     detected in the automatic channelization code search as -10 dB
  //    Define the maximum allowed output power of the UE transmitter as 33 dB
  //    Define the P-CPICH power reported to the UE as 30.8 dB
  //    Define the DPCCH power offset as -80 dB
  //    Define the estimated UL interference level in dBm as -100 dBm
  //    Define a constant offset for the initial preamble power: -10.0 dB
  
  
  /*printf ("\tSelect the channel number of the RF signal analyzed: (9612 to 9888) \n\t"); 
  scanf ("%i", &ue_chan);
  ue_freq = ((ue_chan - 9612)*0.2)+1922.4;
  printf ("\tChannel number set to %i (Frequency: %f MHz)", ue_chan, ue_freq);
  CheckCmuError (RSCMU_WCDMA_Sig_UESigFreq (g_hRsCmuWcdma_Sig, ue_freq, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_UESigChannel (g_hRsCmuWcdma_Sig, ue_chan, errorMessage));*/
  CheckCmuError (RSCMU_WCDMA_Sig_ConfUESigFreqOffs (g_hRsCmuWcdma_Sig, 0, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfSigULSCode (g_hRsCmuWcdma_Sig, 0, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfSigUESigDPCHSyncM (g_hRsCmuWcdma_Sig, 1, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfSigUESigDPCHCorr (g_hRsCmuWcdma_Sig, 0, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfSigUESigDPCHThr (g_hRsCmuWcdma_Sig, -10, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfUESigUEPower (g_hRsCmuWcdma_Sig, 33.0, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfUESigCPICHRepPow (g_hRsCmuWcdma_Sig, 30.8, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfUESigDPCCHPowOffs (g_hRsCmuWcdma_Sig, -80.0, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfUESigOPLCInterfUL (g_hRsCmuWcdma_Sig, -80.0, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfUESigOPLCConstant (g_hRsCmuWcdma_Sig, -29.0, errorMessage));
  fflush (stdin);
  
  
  //-----------------------
  //BSSIGNAL CONFIGURATION: 
  //-----------------------
  //    Define the carrier channel number (frequency corresponding) of the RF signal analyzed  
  //    Define the offset for the RF generator frequency set with the Channel/Frequency function as 0.0 kHz
  //    Define the primary scrambling code for the downlink RF generator signal as 105
  //    Set the absolute output channel power of the BS signal to -50.6 dB
  //    Define all channel power to be expressed relative to the PCPICH
  //    Set the level of the Orthogonal Channel Noise Simulator relative to the P-CPICH level to OFF
  //    Select the dedicated channel to be allocated between the CMU/UTRAN and the UE: Reference Measurement Channel
  //    Set the information bit rate of the 3GPP DL Reference Measurement Channel to 12.2 kbps
  //    Disable the uplink CRC
  //    Set the operating mode that the UE enters after connecting to the UTRAN/CMU to LoopMode 2
  //    Set the bit pattern to be transmitted as user information on the DTCH to "alternating"
  //    Disable the loopback function for voice data
  //    Set the data rate of the SRB to 3.4 kbps
  //    Set the absolute level of the Primary Common Pilot Channel to -55 dB                   
  //    Set the level of the Primary Synchronization Channel relative to the P-CPICH level to -5.0 dB
  //    Set the level of the Secondary Synchronization Channel relative to the P-CPICH level to -5.0 dB
  //    Set the level of the Primary Common Control Physical Channel relative to the PCPICH level to -2.0 dB
  //    Set the level of the Secondary Common Control Physical Channel relative to the PCPICH level to -2.0 dB
  //    Set the level of the Page Indicator Channel relative to the P-CPICH level to -5.0 dB
  //    Set the level of the Acquisition Indicator Channel relative to the P-CPICH level to - 5.0 dB
  //    Set the level of the Dedicated Physical Data Channel relative to the P-CPICH level to -7.0 dB
  //    Assign the channelization code 6 to the Dedicated Physical Data Channel
  //    Set the ratio of DPCCH to DPDCH power to 0.0 dB
  //    Define the secondary scrambling code for the downlink RF generator signal as 0.0 dB
  //    Define the power control algorithm used to control the UE transmit power as Algorithm 2
  //    Define the bit pattern transmitted on the Dedicated Traffic Channel as "alternating"
  //    Set the relative power difference of the UE between two consecutive slots if it is commanded 
  //     to change its output power to 1 dB
  //    Define the slot format number of the UL DPCCH as "0000000000000000000000"
  //    Cause a single TPC pattern (defined via CONFigure:BSSignal:TPC:PATTern) to be sent at the next allowed frame boundary
  
  printf ("\n\n\tSelect the carrier channel of the RF signal analyzed: (10562 to 10838) \n\t"); 
  scanf ("%i", &bs_chan);
  bs_freq = 2112.4+((bs_chan - 10562)*0.2);
  printf ("\tChannel number set to %i (Frequency: %f MHz)", bs_chan, bs_freq);
  CheckCmuError (RSCMU_WCDMA_Sig_BSSigFreq (g_hRsCmuWcdma_Sig, bs_freq, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_BSSigChannel (g_hRsCmuWcdma_Sig, bs_chan, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfBSSigFreqOffset (g_hRsCmuWcdma_Sig, 0.0, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfSigPrimaryScramC (g_hRsCmuWcdma_Sig, 0, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfSigBSSigLevelRef (g_hRsCmuWcdma_Sig, 0, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfSigBSSigOCNS (g_hRsCmuWcdma_Sig, -35.0, 0, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfSigBSSigDCHType (g_hRsCmuWcdma_Sig, 1,
                                                      errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfBSSigDCHRMC_RCh (g_hRsCmuWcdma_Sig, 0, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfSigBSSigDCHUpCRC (g_hRsCmuWcdma_Sig, 0, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfBSSigDCHRMC_TM (g_hRsCmuWcdma_Sig, 3, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfBSSigDCH_DataDTCH (g_hRsCmuWcdma_Sig, 3, errorMessage));
// Function removed //  CheckCmuError (RSCMU_WCDMA_Sig_ConfBSSigDCHEcho (g_hRsCmuWcdma_Sig, 0, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfBSSigDCHSRB (g_hRsCmuWcdma_Sig, 1, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfBSSigPCPICHLevel (g_hRsCmuWcdma_Sig, -55, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfBSSigPSCHLevel (g_hRsCmuWcdma_Sig, -5.0, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfBSSigSSCHLevel (g_hRsCmuWcdma_Sig, -5.0, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfBSSigPCCPCHLevel (g_hRsCmuWcdma_Sig, -2.0, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfBSSigSCCPCHLevel (g_hRsCmuWcdma_Sig, -12.0, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfBSSigPICHLevel (g_hRsCmuWcdma_Sig, -5.0, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfBSSigAICHLevel (g_hRsCmuWcdma_Sig, -5.0, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfBSSigDPDCHLevel (g_hRsCmuWcdma_Sig, -7.0, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfBSSigDPDCHCode (g_hRsCmuWcdma_Sig, 6, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfBSSigPowerOffset (g_hRsCmuWcdma_Sig, 0.0, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfSigSecondScramC (g_hRsCmuWcdma_Sig, 0, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfBSSigTPCAlgor (g_hRsCmuWcdma_Sig, 1, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfBSSigTPCPatterTyp (g_hRsCmuWcdma_Sig, 0, errorMessage));
  //CheckCmuError (RSCMU_WCDMA_Sig_ConfBSSigTPCStepSize (g_hRsCmuWcdma_Sig, 1, errorMessage));
  //CheckCmuError (RSCMU_WCDMA_Sig_ConfBSSigTPCPattern (g_hRsCmuWcdma_Sig, "0000000000000000000000", errorMessage));
  //CheckCmuError (RSCMU_WCDMA_Sig_ConfNetwTPCRepeatPat (g_hRsCmuWcdma_Sig, RSCMU_ON, errorMessage));
  fflush (stdin);
  
  //--------------------------
  //NETWORK CONFIGURATION
  //--------------------------
  //    Define the 3-digit Mobile Country Code as 001
  //    Set the 2-digit Mobile Network Code as 01
  //    Define the Location Area Code as 1
  //    Set the 16-digit URA identity transmitted to the UE for random access as "0000000000000001"
  //    Select the 10 ms RACH Transmission Time Interval
  //    Define the length of a Discontinuous Reception (DRX) cycle as 7
  //    Define the maximum number of times the preamble cycle is repeated as 2
  //    Define the power difference between the last preamble transmitted and the RACH message part as -5.0 dB
  //    Define the transmit power difference between two consecutive preambles as 3.0 dB
  //    Define the minimum time delay between two consecutive preambles as 3 slots
  //    Define the max. number of preambles to be transmitted before a single preamble cycle is terminated as 6
  //    Enable the CMU to request the UEs radio access capability
  //    Enable request of the IMEI number
  
  CheckCmuError (RSCMU_WCDMA_Sig_ConfNetwMCC (g_hRsCmuWcdma_Sig, 1, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfNetwMNC (g_hRsCmuWcdma_Sig, 1, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfNetwLAC (g_hRsCmuWcdma_Sig, 1, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfNetwURAIdentity (g_hRsCmuWcdma_Sig, "0000000000000001", errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfNetwRAC_TTI (g_hRsCmuWcdma_Sig, 0, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfNetwDRXCycle (g_hRsCmuWcdma_Sig, 8, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfNetwRACHPreamCycl (g_hRsCmuWcdma_Sig, 2, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfNetwRACHPOffs (g_hRsCmuWcdma_Sig, -5.0, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfNetwRACHSlotSize (g_hRsCmuWcdma_Sig, 3.0, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfNetwRACHTranTAICH (g_hRsCmuWcdma_Sig, 3, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfNetwRACHMaxRetran (g_hRsCmuWcdma_Sig, 6, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfNetwRequCapabil (g_hRsCmuWcdma_Sig, 1, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfNetwRequIMEI (g_hRsCmuWcdma_Sig, 1, errorMessage));
  
  
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // SIGNALLING SETTINGS
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  //    Set the Signalling State to OFF
  //    Set the Data Rate of the SRB to 3.4 kbit/s
  //    Define the trigger condition as "Free Run"
  
  CheckCmuError (RSCMU_WCDMA_Sig_ProcSignAct (g_hRsCmuWcdma_Sig, RSCMU_WCDMA_SOFF, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_ConfBSSigDCHSRB (g_hRsCmuWcdma_Sig, 1, errorMessage));
  CheckCmuError (RSCMU_WCDMA_Sig_TrigSour (g_hRsCmuWcdma_Sig, RSCMU_FRUN, errorMessage));
  
  
  
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // CALL SETUP
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  
 CheckCmuError (RSCMU_WCDMA_Sig_SensSignStat (g_hRsCmuWcdma_Sig, &signalling_state, errorMessage));
 printf ("\n\n\n\tSwitch on the UE now!\n\n");
 
 //----------------------
 //ESTABLISH CONNECTION
 //----------------------
 
 do
 {
 
 CheckCmuError (RSCMU_WCDMA_Sig_SensSignStat (g_hRsCmuWcdma_Sig, &signalling_state, errorMessage)); 
 
 switch (signalling_state)
       {
 
 case 0:
 
       CheckCmuError (RSCMU_WCDMA_Sig_ProcSignAct (g_hRsCmuWcdma_Sig, RSCMU_WCDMA_SON, errorMessage));
       
       break;
       
 case 1:
        
       
       printf ("\n\n\tWaiting for Registration\n\n");
       
       CheckCmuError (RSCMU_WCDMA_Sig_SensSignStat (g_hRsCmuWcdma_Sig, &registration, errorMessage));
       
       while (l == 0 || l == 1)
         {
         CheckCmuError (RSCMU_WCDMA_Sig_SensSignStat (g_hRsCmuWcdma_Sig, &registration, errorMessage)); 
         l = registration;
         }
       
       break;
       
 case 5:
       
       CheckCmuError (RSCMU_WCDMA_Sig_ProcSignAct (g_hRsCmuWcdma_Sig, RSCMU_WCDMA_CTM, errorMessage));
       
       printf ("\tPaging...\n\n");
       
       CheckCmuError (RSCMU_WCDMA_Sig_SensSignStat (g_hRsCmuWcdma_Sig, &paging, errorMessage));
       
       m = paging;
       
       while (m != 4 && m != 3)
           {
           CheckCmuError (RSCMU_WCDMA_Sig_SensSignStat (g_hRsCmuWcdma_Sig, &paging, errorMessage));
           m = paging;
           }
       
       break;
       
 case 3:
       
       printf ("\tWaiting for UE connection\n\n"); 
       
       CheckCmuError (RSCMU_WCDMA_Sig_SensSignStat (g_hRsCmuWcdma_Sig, &connection, errorMessage));
       
       
       while (n != 6 && n != 3)
           {
           CheckCmuError (RSCMU_WCDMA_Sig_SensSignStat (g_hRsCmuWcdma_Sig, &connection, errorMessage)); 
           n = connection;
           }      
       
       break;
       
 default:
       
       break;    
       
      }
 k = signalling_state;
 }
 while (k != 3);
 
 printf ("\tUE connected\n\n");
 
 
 
 //------------------------
 //OUTPUT POWER SETTING
 //------------------------
 
 printf ("\n\n\tSet output power (dB): (-147.1 to -20.1) \n\t");
 scanf ("%i", &outp_pow);
 CheckCmuError (RSCMU_WCDMA_Sig_ConfSigBSSigOutputPow (g_hRsCmuWcdma_Sig, outp_pow, errorMessage));
 fflush (stdin);
 
 
 //----------------
 //SIGNALLING INFO:
 //----------------
 // Query the signalling state, the registration identity type, the IMEI Number, the number dialed at the UE, 
 //  the number dialed at the CMU to connect to the UE, the PCDP parameters of the UE, the Radio Link Control 
 //  parameters, the parameters of the physical downlink, the parameters of the physical uplink, the Radio Link 
 //  Control parameters of the UE, the multi mode capability of the UE, the Radio Access Technology the UE supports, 
 //  the parameters related to UE positioning, parameters related to inter-frequency or inter-Radio Access Technology 
 //  (RAT) measurements 
 
 CheckCmuError (RSCMU_WCDMA_Sig_SensSignStat (g_hRsCmuWcdma_Sig, &signalling_state, errorMessage));
 CheckCmuError (RSCMU_WCDMA_Sig_SensRIType (g_hRsCmuWcdma_Sig, &registration_identity_type, errorMessage));
 //CheckCmuError (RSCMU_WCDMA_Sig_SensIMEI (g_hRsCmuWcdma_Sig, &imei, errorMessage));
 //CheckCmuError (RSCMU_WCDMA_Sig_SensDialNumber (g_hRsCmuWcdma_Sig, &ue_dialed_number, errorMessage));
 //CheckCmuError (RSCMU_WCDMA_Sig_SensCalNumber (g_hRsCmuWcdma_Sig, &ue_calling_number, errorMessage));
 CheckCmuError (RSCMU_WCDMA_Sig_SensUECapPDCP (g_hRsCmuWcdma_Sig, &lossles_srns_relocation, &rf_2507,
                                               &header_compression, errorMessage));
 CheckCmuError (RSCMU_WCDMA_Sig_SensUECapRLC (g_hRsCmuWcdma_Sig, &am_buffer_size, &max_rlc_window_size,
                                              &max_am_entities, errorMessage));
 CheckCmuError (RSCMU_WCDMA_Sig_SensUECapPhyDownl (g_hRsCmuWcdma_Sig, &sim_transport_ch_d, &sim_cctrch_d, &tti_trans_blocks_d,
                                                   &number_of_tfc_d, &number_of_tf_d, &turbo_decoding_d, &rec_bits_all_d,
                                                   &rec_bits_conv_coded_d, &rec_bits_turbo_coded_d, &dpch_codes_d,
                                                   &physical_chn_bits_d, &sf_512_d, errorMessage));
 CheckCmuError (RSCMU_WCDMA_Sig_SensUECapPhyUpl (g_hRsCmuWcdma_Sig, &sim_transport_ch_u, &sim_cctrch_u, &tti_trans_blocks_u,
                                                  &number_of_tfc_u, &number_of_tf_u, &turbo_decoding_u, &rec_bits_all_u,
                                                  &rec_bits_conv_coded_u, &rec_bits_turbo_coded_u, &dpdch_bits_per_10ms, &pcpch,
                                                  &ue_power_class, errorMessage));
 CheckCmuError (RSCMU_WCDMA_Sig_SensUECapTXRXFrequSep (g_hRsCmuWcdma_Sig, &freq_separation, errorMessage));
 CheckCmuError (RSCMU_WCDMA_Sig_SensUECapMultiCarMode (g_hRsCmuWcdma_Sig, &utra_fdd_tdd, errorMessage));
 CheckCmuError (RSCMU_WCDMA_Sig_SensUECapMultirate (g_hRsCmuWcdma_Sig, &gsm, &multi_carr_mode, errorMessage));
 CheckCmuError (RSCMU_WCDMA_Sig_SensUECapMeasRelated (g_hRsCmuWcdma_Sig, &dl_compressed_mode, &ul_compressed_mode, errorMessage)); 
  
  
  
  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // SHOW SETTINGS IN STANDARD I/O WINDOW
  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  
  printf ("\n\tSignalling Info:\n");
  printf ("\t--------------------\n");
  
  WriteCmuViInt32Value ("\t Signalling State: ", signalling_state); 
  WriteCmuViInt32Value ("\t Registration Identity Type: ", registration_identity_type);
  //printf ("\t IMEI Number = %s", imei);
  //printf ("\t Number Dialed at the UE  = %s", ue_dialed_number);
  //printf ("\t Number Dialed at the CMU  = %s", ue_calling_number);
  WriteCmuViInt32Value ("\t Support of Lossless SRNS Relocation? ", lossles_srns_relocation);
  WriteCmuViInt32Value ("\t Support of RFC 2507? ", rf_2507);
  WriteCmuViInt32Value ("\t Header Compression = ", header_compression); 
  WriteCmuViInt32Value ("\t AM Buffer Size = ", am_buffer_size); 
  WriteCmuViInt32Value ("\t Maximum RLC Window Size = ", max_rlc_window_size);
  WriteCmuViInt32Value ("\t Maximum AM Entities = ", max_am_entities);
  WriteCmuViInt32Value ("\t Simult. Transport Channels (Downlink): ", sim_transport_ch_d);
  WriteCmuViInt32Value ("\t Simult. CCTr. Channel (Downlink): ", sim_cctrch_d);
  WriteCmuViInt32Value ("\t TTI Transport Blocks (Downlink): ", tti_trans_blocks_d);
  WriteCmuViInt32Value ("\t Number Of TFC (Downlink): ", number_of_tfc_d);
  WriteCmuViInt32Value ("\t Number Of TF (Downlink): ", number_of_tf_d); 
  WriteCmuViInt32Value ("\t Turbo Decoding (Downlink)? ", turbo_decoding_d); 
  WriteCmuViInt32Value ("\t Received Bits All (Downlink): ", rec_bits_all_d);
  WriteCmuViInt32Value ("\t Received Bits Conv. Coded (Downlink): ", rec_bits_conv_coded_d);
  WriteCmuViInt32Value ("\t Received Bits Turbo Coded (Downlink) = ", rec_bits_turbo_coded_d);
  WriteCmuViInt32Value ("\t DPCH/PDSCH Codes (Downlink): ", dpch_codes_d);
  WriteCmuViInt32Value ("\t Physical Channel Bits (Downlink): ", physical_chn_bits_d);
  WriteCmuViInt32Value ("\t Support of SF 512? ", sf_512_d);
  //WriteCmuViInt32Value ("\t Support of PDSCH? ", pdsch_d); 
  //WriteCmuViInt32Value ("\t Support of Channel Estimation Pilots? ", chn_est_pilots_d); 
  //WriteCmuViInt32Value ("\t Simultaneous Reception of SCCPCH snd DPCH? ", sccpch_dpch_d);
  //WriteCmuViInt32Value ("\t Simultaneous Reception of SCCPCH, DPCH and PDSCH? ", sccpch_dpch_pdsch_d);
  //WriteCmuViInt32Value ("\t Simultaneous Reception of SCCPCH radio links? ", sccpch_radio_links_d);
  WriteCmuViInt32Value ("\t Simult. Transport Channels (Uplink): ", sim_transport_ch_u);
  WriteCmuViInt32Value ("\t Simult. CCTr. Channel (Uplink): ", sim_cctrch_u);
  WriteCmuViInt32Value ("\t TTI Transport Blocks (Uplink): ", tti_trans_blocks_u);
  WriteCmuViInt32Value ("\t Number Of TFC (Uplink): ", number_of_tfc_u);
  WriteCmuViInt32Value ("\t Number Of TF (Uplink): ", number_of_tf_u); 
  WriteCmuViInt32Value ("\t Turbo Decoding (Uplink)? ", turbo_decoding_u); 
  WriteCmuViInt32Value ("\t Received Bits All (Uplink): ", rec_bits_all_u);
  WriteCmuViInt32Value ("\t Received Bits Conv. Coded (Uplink): ", rec_bits_conv_coded_u);
  WriteCmuViInt32Value ("\t Received Bits Turbo Coded (Uplink) = ", rec_bits_turbo_coded_u);
  WriteCmuViInt32Value ("\t DPDCH Bits per 10 ms: ", dpdch_bits_per_10ms);
  WriteCmuViInt32Value ("\t Physical Channel FDD PCPCH? ", pcpch);
  WriteCmuViInt32Value ("\t UE Power Class: ", ue_power_class);
  WriteCmuViInt32Value ("\t Tx/Rx Frequency Separation: ", freq_separation); 
  WriteCmuViInt32Value ("\t UTRA FDD/TDD? ", utra_fdd_tdd);
  WriteCmuViInt32Value ("\t GSM Support? ", gsm); 
  WriteCmuViInt32Value ("\t Multi-Carrier Mode Support? ", multi_carr_mode);
  WriteCmuViInt32Value ("\t Support of DL Compressed Mode? ", dl_compressed_mode);
  WriteCmuViInt32Value ("\t Support of UL Compressed Mode? ", ul_compressed_mode);
  
  
  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  //MAXIMUM POWER MEASUREMENT 
  //
  //  Define the upper limits for the peak and RMS- averaged UE output power 
  //  Define the lower limits for the peak and RMS- averaged UE output power 
  //  Define the rated values for the peak and RMS- averaged UE output power 
  //  Define the number of slots forming a statistics cycle
  //  Set the number of statistics cycles, the stop condition and the stepping mode for the 
  //   measurement to 100, NONE and STEP
  //  Define the event generated when the measurement is terminated or stopped as "Service request"
  //  Start the Power Measurement
  //  Stop the measurement
  //  Read out the current status of the measurement
  //  Continue the measurement
  //  Read out the measurement result
  //  Abort the Power Measurement
  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
  printf ("%s", "\n\nMaximum Power Measurement? (y/n)");
 
  scanf("%c",&b);
  
  if (b == 'y')
    {
    printf ("%s", "\n\nSet the values (dB) for\n\n\tUpper Limit Peak (0.0 to 5.0): ");
    scanf ("%lf", &upp_lim_peak);
    printf ("%s", "\nUpper Limit RMS (0.0 to 5.0): ");
    scanf ("%lf", &upp_lim_rms);
    printf ("%s", "\nLower Limit Peak (-5.0 to 0.0): ");
    scanf ("%lf", &low_lim_peak);
    printf ("%s", "\nLower Limit RMS (-5.0 to 0.0): ");
    scanf ("%lf", &low_lim_rms);
    printf ("%s", "\nRated Value Peak (-60.0 dBm to +53.0 dBm): ");
    scanf ("%lf", &rat_val_peak);
    printf ("%s", "\nRated Value RMS (-60.0 dBm to +53.0 dBm): ");
    scanf ("%lf", &rat_val_rms);
    
  CheckCmuError (RSCMU_WCDMA_NSig_ConfPowContStat (g_hRsCmuWcdma_Sig, RSCMU_POWER_MAX, 10, errorMessage));
  CheckCmuError (RSCMU_WCDMA_NSig_ConfPowContRep (g_hRsCmuWcdma_Sig, RSCMU_POWER_MAX, RSCMU_SINGLE_SHOT, 100,
                                                  RSCMU_NONE, RSCMU_NONE, errorMessage));
  CheckCmuError (RSCMU_WCDMA_NSig_ConfPowMaxLimUpp (g_hRsCmuWcdma_Sig, RSCMU_LIMIT_CMM, upp_lim_peak, upp_lim_rms, errorMessage));
  CheckCmuError (RSCMU_WCDMA_NSig_ConfPowMaxLimLow (g_hRsCmuWcdma_Sig, RSCMU_LIMIT_CMM, low_lim_peak, low_lim_rms,
                                                    errorMessage));
  CheckCmuError (RSCMU_WCDMA_NSig_ConfPowMaxLimRated (g_hRsCmuWcdma_Sig, RSCMU_LIMIT_CMM, rat_val_peak, rat_val_rms, errorMessage));
  CheckCmuError (RSCMU_WCDMA_NSig_ConfPowErep (g_hRsCmuWcdma_Sig, RSCMU_POWER_MAX, RSCMU_SRQ, errorMessage));
  CheckCmuError (RSCMU_WCDMA_NSig_InitPow (g_hRsCmuWcdma_Sig, RSCMU_POWER_MAX, errorMessage));
  CheckCmuError (RSCMU_WCDMA_NSig_StopPow (g_hRsCmuWcdma_Sig, RSCMU_POWER_MAX, errorMessage)); 
  CheckCmuError (RSCMU_WCDMA_NSig_FetcPowStat (g_hRsCmuWcdma_Sig, RSCMU_POWER_MAX, &measurement_status_maxpow, &counter_cycle_maxpow,
                                               &counter_evalperiod_maxpow, errorMessage));
  CheckCmuError (RSCMU_WCDMA_NSig_ContPow (g_hRsCmuWcdma_Sig, RSCMU_POWER_MAX, errorMessage));
  CheckCmuError (RSCMU_WCDMA_NSig_FetcPow (g_hRsCmuWcdma_Sig, RSCMU_POWER_MAX, &Peak_Power_Current, &Peak_Power_Average,
                                           &Peak_Power_Maximum, &RMS_Power_Current, &RMS_Power_Average, &RMS_Power_Maximum,
                                           &RMS_Power_Minimum, &Out_Of_Tolerance, errorMessage));
  CheckCmuError (RSCMU_WCDMA_NSig_AborPow (g_hRsCmuWcdma_Sig, RSCMU_POWER_MAX, errorMessage));
  
  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // SHOW RESULTS IN STANDARD I/O WINDOW
  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  
  printf ("\n\tMaximum Power Measurement\n");
  printf ("\t-----------------------------\n");
  
  WriteCmuViReal64Value ("\t Current Peak Power = ", Peak_Power_Current); 
  WriteCmuViReal64Value ("\t Average Peak Power = ", Peak_Power_Average);
  WriteCmuViReal64Value ("\t Maximum Peak Power = ", Peak_Power_Maximum);
  WriteCmuViReal64Value ("\t Current RMS Power  = ", RMS_Power_Current);
  WriteCmuViReal64Value ("\t Average RMS Power  = ", RMS_Power_Average);
  WriteCmuViReal64Value ("\t Maximum RMS Power  = ", RMS_Power_Maximum);
  WriteCmuViReal64Value ("\t Minimum RMS Power  = ", RMS_Power_Minimum);
  WriteCmuViReal64Value ("\t Out Of Tolerance   = ", Out_Of_Tolerance);
  };
  fflush (stdin);
  
/*  
  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  //MODULATION WCDMA OVERVIEW MEASUREMENT CONFIGURATION
  //   Set all values to default
  //   Set channelization codes to Manual input
  //   Define the number of slots forming a statistics cycle as 10
  //   Set the number of statistics cycles, the stop condition and the stepping mode for the measurement to 100, NONE and Continuous
  //   Define upper limits for the Current and Max./Min traces and for the scalar modulation parameters derived from them.
  //   Define the event generated when the measurement is terminated or stopped as "single operation complete"
  //   Start a new Measurement
  //   Read out the current status of the measurement
  //   Read out the results of the measurement
  //   Calculate whether and in which way the error limits for the scalar measured values have been exceeded
  //   Abort the measurement
  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  
  printf ("%s", "\n\nModulation WCDMA Overview? (y/n)");
 
  scanf("%c",&c);
  
  if (c == 'y')
    {
  CheckCmuError (RSCMU_WCDMA_NSig_DefOverWcdmLim (g_hRsCmuWcdma_Sig, RSCMU_DPCH, RSCMU_ON, errorMessage));
  CheckCmuError (RSCMU_WCDMA_NSig_ConfModWcdmContCcm (g_hRsCmuWcdma_Sig, RSCMU_DPCH, RSCMU_AUTO, errorMessage));

  
    //CheckCmuError (RSCMU_WCDMA_NSig_ConfOverWcdmContStat (g_hRsCmuWcdma_Sig, RSCMU_DPCH, 10, errorMessage));
    //CheckCmuError (RSCMU_WCDMA_NSig_ConfOverWcdmContRep (g_hRsCmuWcdma_Sig, RSCMU_DPCH, RSCMU_SINGLE_SHOT, 100,
    //                                                 RSCMU_NONE, RSCMU_NONE, errorMessage));
    //CheckCmuError (RSCMU_WCDMA_NSig_ConfOverWcdmLimUpp (g_hRsCmuWcdma_Sig, RSCMU_DPCH, RSCMU_LIMIT_CMM, 17.5, 17.5, 17.5, 17.5,
    //                                                10.0, 10.0, -25.0, -15.0, 200, 0.944, -15.0, 0.06, errorMessage));
    //CheckCmuError (RSCMU_WCDMA_NSig_ConfOverWcdmErep (g_hRsCmuWcdma_Sig, RSCMU_DPCH, RSCMU_SOPC, errorMessage));
    
  
  
  
  CheckCmuError (RSCMU_WCDMA_NSig_InitOverWcdm (g_hRsCmuWcdma_Sig, RSCMU_DPCH, errorMessage));
  CheckCmuError (RSCMU_WCDMA_NSig_FetcOverWcdmStat (g_hRsCmuWcdma_Sig, RSCMU_DPCH, &measurement_status_wcdma,
                                                    &counter_cycle_wcdma, &counter_evalperiod_wcdma, errorMessage));
  CheckCmuError (RSCMU_WCDMA_NSig_FetcOverWcdm (g_hRsCmuWcdma_Sig, RSCMU_DPCH, RSCMU_AVERAGE, &evm_peak_wcdma, &evm_rms_wcdma, &magn_error_wcdma,
                                                &magn_error_rms_wcdma, &phase_error_peak_wcdma, &phase_error_rms_wcdma, &iq_origin_offset_wcdma,
                                                &iq_imbalance_wcdma, &freq_err_wcdma, &wav_quality_wcdma, &peak_cd_err_wcdma,
                                                &transmit_time_err_wcdma, &ue_pow_curr_wcdma, &out_of_tolerance_wcdma, &slot_nr_wcdma, errorMessage));
  CheckCmuError (RSCMU_WCDMA_NSig_CalcOverWcdmMatch (g_hRsCmuWcdma_Sig, RSCMU_DPCH, RSCMU_CURRENT, &error_vector_magn_peak_wcdma_matc,
                                                     &error_vector_magn_rms_wcdma_matc, &magn_err_peak_wcdma_matc, &magn_err_rms_wcdma_matc,
                                                     &phase_err_peak_wcdma_matc, &phase_err_rms_wcdma_matc, &iq_origin_offset_wcdma_matc,
                                                     &iq_imbalance_wcdma_matc, &freq_err_wcdma_matc, &wav_qual_wcdma_matc, &peak_cd_err_wcdma_matc,
                                                     &transmit_time_err_wcdma_matc, errorMessage));
  CheckCmuError (RSCMU_WCDMA_NSig_AborOverWcdm (g_hRsCmuWcdma_Sig, RSCMU_DPCH, errorMessage));
  
  printf ("\n\tWCDMA Modulation Overview Measurement\n");
  printf ("\t-----------------------------------------\n");
  
  WriteCmuViReal64Value ("\t EVM Peak               = ", evm_peak_wcdma);
  WriteCmuViReal64Value ("\t EVM RMS                = ", evm_rms_wcdma);
  WriteCmuViReal64Value ("\t Magnitude Error Peak   = ", magn_error_wcdma);
  WriteCmuViReal64Value ("\t Magnitude Error RMS    = ", magn_error_rms_wcdma);
  WriteCmuViReal64Value ("\t Phase Error Peak       = ", phase_error_peak_wcdma);
  WriteCmuViReal64Value ("\t Phase Error RMS        = ", phase_error_rms_wcdma);
  WriteCmuViReal64Value ("\t I/Q Origin Offset      = ", iq_origin_offset_wcdma);
  WriteCmuViReal64Value ("\t I/Q Imbalance      = ", iq_imbalance_wcdma);
  WriteCmuViReal64Value ("\t Frequency Error    = ", freq_err_wcdma);
  WriteCmuViReal64Value ("\t Waveform Quality   = ", wav_quality_wcdma);
  WriteCmuViReal64Value ("\t Peak Code Domain Error = ", peak_cd_err_wcdma);
  WriteCmuViReal64Value ("\t Transmit Time Error    = ", transmit_time_err_wcdma);  
  WriteCmuViReal64Value ("\t Current UE Power       = ", ue_pow_curr_wcdma);
  WriteCmuViReal64Value ("\t Out Of Tolerance   = ", out_of_tolerance_wcdma);
  WriteCmuViInt32Value ("\t Slot Number     = ", slot_nr_wcdma);
  
  printf ("\n\tMatching Limits?\n");
  printf ("\t-------------------\n");
  
  WriteCmuViInt32Value ("\t EVM Peak               : ", error_vector_magn_peak_wcdma_matc);
  WriteCmuViInt32Value ("\t EVM RMS                : ", error_vector_magn_rms_wcdma_matc);
  WriteCmuViInt32Value ("\t Magnitude Error Peak   : ", magn_err_peak_wcdma_matc);
  WriteCmuViInt32Value ("\t Magnitude Error RMS    : ", magn_err_rms_wcdma_matc);
  WriteCmuViInt32Value ("\t Phase Error Peak       : ", phase_err_peak_wcdma_matc);
  WriteCmuViInt32Value ("\t Phase Error RMS        : ", phase_err_rms_wcdma_matc);
  WriteCmuViInt32Value ("\t I/Q Origin Offset      : ", iq_origin_offset_wcdma_matc);
  WriteCmuViInt32Value ("\t I/Q Imbalance           : ", iq_imbalance_wcdma_matc);
  WriteCmuViInt32Value ("\t Frequency Error : ", freq_err_wcdma_matc);
  WriteCmuViInt32Value ("\t Waveform Quality    : ", wav_qual_wcdma_matc);
  WriteCmuViInt32Value ("\t Peak Code Domain Error : ", peak_cd_err_wcdma_matc);
  WriteCmuViInt32Value ("\t Transmit Time Error    : ", transmit_time_err_wcdma_matc);
  };
  fflush (stdin);
  
  
 */
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 // RECEIVER QUALITY MEASUREMENTS
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //  Set Default values for Reciever Quality subsystem and for the limits
 //  Set the event generated when the measurement is terminated or stopped to "service request"
 //  Set the type of repetition, number of statistics cycles, the stop condition and the stepping mode for the 
 //   measurement to "single shot", 1000, NONE and NONE
 //  Set the upper limits for the Bit Error Rate, the Block Error Rate and the Data Block Error Rate manually
 //  Select the output power to start the measurement with
 //  Select the step size the output power is gradually decreased by during the measurement
 //  Initialize the measurement
 //  Continue the measurement until all three limits have been at least once exceeded  
 //  Abort the measurement
  
  printf ("%s", "\n\nReceiver Quality Measurement? (y/n)");
 
  scanf("%c",&g);
  
  if (g == 'y')
    {
    CheckCmuError (RSCMU_WCDMA_NSig_DefRXQuBERCont (g_hRsCmuWcdma_Sig, RSCMU_ON, errorMessage));
    CheckCmuError (RSCMU_WCDMA_NSig_ConfRXQuBERContRep (g_hRsCmuWcdma_Sig, RSCMU_SINGLE_SHOT, 1000, RSCMU_NONE,
                                                        RSCMU_NONE, errorMessage));
    CheckCmuError (RSCMU_WCDMA_NSig_DefRXQuBERLim (g_hRsCmuWcdma_Sig, RSCMU_ON, errorMessage));
    CheckCmuError (RSCMU_WCDMA_NSig_ConfRXQuBERTBLocks (g_hRsCmuWcdma_Sig, 200, errorMessage));
    
    printf ("%s", "\n\n\tSet Upper Limit for BER (%)\n\t");
    scanf("%f",&ber_lim);
    fflush (stdin);
    printf ("%s", "\n\n\tSet Upper Limit for BLER (%)\n\t");
    scanf ("%f", &bler_lim);
    fflush (stdin);
    printf ("%s", "\n\n\tSet Upper Limit for DBLER (%)\n\t");
    scanf ("%f", &dbler_lim);
    fflush (stdin);
    CheckCmuError (RSCMU_WCDMA_NSig_ConfRXQuBERLimit (g_hRsCmuWcdma_Sig, ber_lim, bler_lim, dbler_lim, errorMessage));
    CheckCmuError (RSCMU_WCDMA_NSig_ConfRXQuBERErep (g_hRsCmuWcdma_Sig, RSCMU_SRQ, errorMessage));
    CheckCmuError (RSCMU_WCDMA_NSig_InitRXQuBER (g_hRsCmuWcdma_Sig, errorMessage));
    CheckCmuError (RSCMU_WCDMA_NSig_FetcRXQuBERStat (g_hRsCmuWcdma_Sig, &meas_stat_BER, &counter_stat_cycle_BER,
                                                     &counter_eval_period_BER, errorMessage));
    
    printf ("%s", "\n\n\tSelect the output channel power to start with (dB)\n\t");
    scanf ("%f", &ber_power);
    CheckCmuError (RSCMU_WCDMA_Sig_ConfSigBSSigOutputPow (g_hRsCmuWcdma_Sig, ber_power, errorMessage));
    fflush (stdin);
    printf ("%s", "\n\n\tSelect the step size the power is decreased by (dB)\n\t");
    scanf ("%f", &step_size_ber);
    fflush (stdin);
    do
    {
    CheckCmuError (RSCMU_WCDMA_Sig_ConfSigBSSigOutputPow (g_hRsCmuWcdma_Sig, ber_power, errorMessage));
    CheckCmuError (RSCMU_WCDMA_NSig_ReadRXQuBER (g_hRsCmuWcdma_Sig, &ber, &bler, &dbler, &dl_ul_alignment,
                                                 &lost_transp_blocks, errorMessage));
    CheckCmuError (RSCMU_WCDMA_NSig_CalcRCQuBERMatch (g_hRsCmuWcdma_Sig, &BER_lim_match, &BLER_lim_match,
                                                      &DBLER_lim_match, errorMessage));
    ber_power = ber_power - step_size_ber; 
    }
    while (BER_lim_match == 0 && BLER_lim_match == 0 && DBLER_lim_match == 0);
    
    //-----------------------------------------------------------------------
    if (BER_lim_match != 0 && BLER_lim_match != 0 && DBLER_lim_match !=0)
      {
      printf ("%s %f", "\n\n\tBER, BLER and DBLER limits violated at output power: ", ber_power);
      ber_pow_lim = ber_power;
      bler_pow_lim = ber_power;
      dbler_pow_lim = ber_power;
      }
      else
      {
      //---------------------------------------------------------------------
      if (BER_lim_match != 0 && BLER_lim_match != 0)
        {
        printf ("%s %f", "\n\n\tBER and BLER limits violated at output power: ", ber_power);
        ber_pow_lim = ber_power;
        bler_pow_lim = ber_power;
        do
        {
        CheckCmuError (RSCMU_WCDMA_Sig_ConfSigBSSigOutputPow (g_hRsCmuWcdma_Sig, ber_power, errorMessage));
        CheckCmuError (RSCMU_WCDMA_NSig_ReadRXQuBER (g_hRsCmuWcdma_Sig, &ber, &bler, &dbler, &dl_ul_alignment,
                                                 &lost_transp_blocks, errorMessage));
        CheckCmuError (RSCMU_WCDMA_NSig_CalcRCQuBERMatch (g_hRsCmuWcdma_Sig, &BER_lim_match, &BLER_lim_match,
                                                      &DBLER_lim_match, errorMessage));
        ber_power1 = ber_power;
        ber1 = ber;
        bler1 = bler;
        dbler1 = dbler;
        dl_ul_alignment1 = dl_ul_alignment;
        lost_transport_blocks1 = lost_transp_blocks;
        BER_lim_match1 = BER_lim_match;
        BLER_lim_match1 = BLER_lim_match;
        DBLER_lim_match1 = DBLER_lim_match;
        
        WriteCmuViInt32Value ("\n\n\t Measurement Status     = ", meas_stat_BER);
        WriteCmuViInt32Value ("\t Counter stat. cycles   = ", counter_stat_cycle_BER);
        WriteCmuViInt32Value ("\t Counter eval. periods     = ", counter_eval_period_BER);
        WriteCmuViReal64Value ("\t Bit Error Rate        = ", ber);
        WriteCmuViReal64Value ("\t Block Error Rate     = ", bler);
        WriteCmuViReal64Value ("\t Data Block Error Rate  = ", dbler);
        WriteCmuViInt32Value ("\t DL/UL Alignment       = ", dl_ul_alignment);
        WriteCmuViInt32Value ("\t Lost Transport Blocks = ", lost_transp_blocks);
  
        printf ("\n\tMatching Limits?\n");
        printf ("\t-------------------\n");
  
        WriteCmuViInt32Value ("\t Bit Error Rate          : ", BER_lim_match);
        WriteCmuViInt32Value ("\t Block Error Rate        : ", BLER_lim_match);
        WriteCmuViInt32Value ("\t Data Block Error Rate   : ", DBLER_lim_match);
        
        
        ber_power = ber_power - step_size_ber; 
        }
        while (DBLER_lim_match == 0);
        printf ("%s %f", "\n\n\tDBLER limit violated at output power: ", ber_power);
        dbler_pow_lim = ber_power + step_size_ber;
        }
        else
        {
        //-------------------------------------------------------------------
        if (BER_lim_match != 0 && DBLER_lim_match != 0)
          {
          printf ("%s %f", "\n\n\tBER and DBLER limits violated at output power: ", ber_power);
          ber_pow_lim = ber_power+step_size_ber;
          dbler_pow_lim = ber_power+step_size_ber;
          do
          {
          CheckCmuError (RSCMU_WCDMA_Sig_ConfSigBSSigOutputPow (g_hRsCmuWcdma_Sig, ber_power, errorMessage));
          CheckCmuError (RSCMU_WCDMA_NSig_ReadRXQuBER (g_hRsCmuWcdma_Sig, &ber, &bler, &dbler, &dl_ul_alignment,
                                                 &lost_transp_blocks, errorMessage));
          CheckCmuError (RSCMU_WCDMA_NSig_CalcRCQuBERMatch (g_hRsCmuWcdma_Sig, &BER_lim_match, &BLER_lim_match,
                                                      &DBLER_lim_match, errorMessage));
          ber_power1 = ber_power;
          ber1 = ber;
          bler1 = bler;
          dbler1 = dbler;
          dl_ul_alignment1 = dl_ul_alignment;
          lost_transport_blocks1 = lost_transp_blocks;
          BER_lim_match1 = BER_lim_match;
          BLER_lim_match1 = BLER_lim_match;
          DBLER_lim_match1 = DBLER_lim_match;
          
          WriteCmuViInt32Value ("\n\n\t Measurement Status     = ", meas_stat_BER);
          WriteCmuViInt32Value ("\t Counter stat. cycles   = ", counter_stat_cycle_BER);
          WriteCmuViInt32Value ("\t Counter eval. periods   = ", counter_eval_period_BER);
          WriteCmuViReal64Value ("\t Bit Error Rate        = ", ber);
          WriteCmuViReal64Value ("\t Block Error Rate       = ", bler);
          WriteCmuViReal64Value ("\t Data Block Error Rate  = ", dbler);
          WriteCmuViInt32Value ("\t DL/UL Alignment         = ", dl_ul_alignment);
          WriteCmuViInt32Value ("\t Lost Transport Blocks   = ", lost_transp_blocks);
  
          printf ("\n\tMatching Limits?\n");
          printf ("\t-------------------\n");
  
          WriteCmuViInt32Value ("\t Bit Error Rate          : ", BER_lim_match);
          WriteCmuViInt32Value ("\t Block Error Rate        : ", BLER_lim_match);
          WriteCmuViInt32Value ("\t Data Block Error Rate   : ", DBLER_lim_match);
          
          
          
          ber_power = ber_power - step_size_ber; 
          }
          while (BLER_lim_match == 0);
          printf ("%s %f", "\n\n\tBLER limit violated at output power: ", ber_power);
          bler_pow_lim = ber_power+step_size_ber;
          }
          else
          {
          //-----------------------------------------------------------------
          if (BLER_lim_match != 0 && DBLER_lim_match != 0)
            {
            printf ("%s %f", "\n\n\tBLER and DBLER limits violated at output power: ", ber_power);
            bler_pow_lim = ber_power+step_size_ber;
            dbler_pow_lim = ber_power+step_size_ber;
            do
            {
            CheckCmuError (RSCMU_WCDMA_Sig_ConfSigBSSigOutputPow (g_hRsCmuWcdma_Sig, ber_power, errorMessage));
            CheckCmuError (RSCMU_WCDMA_NSig_ReadRXQuBER (g_hRsCmuWcdma_Sig, &ber, &bler, &dbler, &dl_ul_alignment,
                                                 &lost_transp_blocks, errorMessage));
            CheckCmuError (RSCMU_WCDMA_NSig_CalcRCQuBERMatch (g_hRsCmuWcdma_Sig, &BER_lim_match, &BLER_lim_match,
                                                      &DBLER_lim_match, errorMessage));
            ber_power1 = ber_power;
            ber1 = ber;
            bler1 = bler;
            dbler1 = dbler;
            dl_ul_alignment1 = dl_ul_alignment;
            lost_transport_blocks1 = lost_transp_blocks;
            BER_lim_match1 = BER_lim_match;
            BLER_lim_match1 = BLER_lim_match;
            DBLER_lim_match1 = DBLER_lim_match;
            
            WriteCmuViInt32Value ("\n\n\t Measurement Status     = ", meas_stat_BER);
            WriteCmuViInt32Value ("\t Counter stat. cycles   = ", counter_stat_cycle_BER);
            WriteCmuViInt32Value ("\t Counter eval. periods     = ", counter_eval_period_BER);
            WriteCmuViReal64Value ("\t Bit Error Rate        = ", ber);
            WriteCmuViReal64Value ("\t Block Error Rate     = ", bler);
            WriteCmuViReal64Value ("\t Data Block Error Rate  = ", dbler);
            WriteCmuViInt32Value ("\t DL/UL Alignment       = ", dl_ul_alignment);
            WriteCmuViInt32Value ("\t Lost Transport Blocks = ", lost_transp_blocks);
  
            printf ("\n\tMatching Limits?\n");
            printf ("\t-------------------\n");
  
            WriteCmuViInt32Value ("\t Bit Error Rate          : ", BER_lim_match);
            WriteCmuViInt32Value ("\t Block Error Rate        : ", BLER_lim_match);
            WriteCmuViInt32Value ("\t Data Block Error Rate   : ", DBLER_lim_match);
            
            
            
            ber_power = ber_power - step_size_ber; 
            } 
            while (BER_lim_match == 0);
            printf ("%s %f", "\n\n\tBER limit violated at output power: ", ber_power);
            ber_pow_lim = ber_power+step_size_ber;
            }
            else
            {
            //---------------------------------------------------------------
            if (BER_lim_match != 0)
              {
              printf ("%s %f", "\n\n\tBER limit violated at output power: ", ber_power);
              ber_pow_lim = ber_power+step_size_ber;
              do
              {
              CheckCmuError (RSCMU_WCDMA_Sig_ConfSigBSSigOutputPow (g_hRsCmuWcdma_Sig, ber_power, errorMessage));
              CheckCmuError (RSCMU_WCDMA_NSig_ReadRXQuBER (g_hRsCmuWcdma_Sig, &ber, &bler, &dbler, &dl_ul_alignment,
                                                 &lost_transp_blocks, errorMessage));
              CheckCmuError (RSCMU_WCDMA_NSig_CalcRCQuBERMatch (g_hRsCmuWcdma_Sig, &BER_lim_match, &BLER_lim_match,
                                                      &DBLER_lim_match, errorMessage));
              ber_power1 = ber_power;
              ber1 = ber;
              bler1 = bler;
              dbler1 = dbler;
              dl_ul_alignment1 = dl_ul_alignment;
              lost_transport_blocks1 = lost_transp_blocks;
              BER_lim_match1 = BER_lim_match;
              BLER_lim_match1 = BLER_lim_match;
              DBLER_lim_match1 = DBLER_lim_match;
              
              WriteCmuViInt32Value ("\n\n\t Measurement Status     = ", meas_stat_BER);
              WriteCmuViInt32Value ("\t Counter stat. cycles   = ", counter_stat_cycle_BER);
              WriteCmuViInt32Value ("\t Counter eval. periods   = ", counter_eval_period_BER);
              WriteCmuViReal64Value ("\t Bit Error Rate        = ", ber);
              WriteCmuViReal64Value ("\t Block Error Rate       = ", bler);
              WriteCmuViReal64Value ("\t Data Block Error Rate  = ", dbler);
              WriteCmuViInt32Value ("\t DL/UL Alignment         = ", dl_ul_alignment);
              WriteCmuViInt32Value ("\t Lost Transport Blocks   = ", lost_transp_blocks);
  
              printf ("\n\tMatching Limits?\n");
              printf ("\t-------------------\n");
  
              WriteCmuViInt32Value ("\t Bit Error Rate          : ", BER_lim_match);
              WriteCmuViInt32Value ("\t Block Error Rate        : ", BLER_lim_match);
              WriteCmuViInt32Value ("\t Data Block Error Rate   : ", DBLER_lim_match);
              
              
              
              ber_power = ber_power - step_size_ber; 
              } 
              while (BLER_lim_match == 0 && DBLER_lim_match == 0);
              if (BLER_lim_match != 0)
                {
                printf ("%s %f", "\n\n\tBLER limit violated at output power: ", ber_power);
                bler_pow_lim = ber_power+step_size_ber;
                do
                {
                CheckCmuError (RSCMU_WCDMA_Sig_ConfSigBSSigOutputPow (g_hRsCmuWcdma_Sig, ber_power, errorMessage));
                CheckCmuError (RSCMU_WCDMA_NSig_ReadRXQuBER (g_hRsCmuWcdma_Sig, &ber, &bler, &dbler, &dl_ul_alignment,
                                                 &lost_transp_blocks, errorMessage));
                CheckCmuError (RSCMU_WCDMA_NSig_CalcRCQuBERMatch (g_hRsCmuWcdma_Sig, &BER_lim_match, &BLER_lim_match,
                                                      &DBLER_lim_match, errorMessage));
                ber_power2 = ber_power;
                ber2 = ber;
                bler2 = bler;
                dbler2 = dbler;
                dl_ul_alignment2 = dl_ul_alignment;
                lost_transport_blocks2 = lost_transp_blocks;
                BER_lim_match2 = BER_lim_match;
                BLER_lim_match2 = BLER_lim_match;
                DBLER_lim_match2 = DBLER_lim_match;
                
                WriteCmuViInt32Value ("\n\n\t Measurement Status     = ", meas_stat_BER);
                WriteCmuViInt32Value ("\t Counter stat. cycles   = ", counter_stat_cycle_BER);
                WriteCmuViInt32Value ("\t Counter eval. periods     = ", counter_eval_period_BER);
                WriteCmuViReal64Value ("\t Bit Error Rate        = ", ber);
                WriteCmuViReal64Value ("\t Block Error Rate     = ", bler);
                WriteCmuViReal64Value ("\t Data Block Error Rate  = ", dbler);
                WriteCmuViInt32Value ("\t DL/UL Alignment       = ", dl_ul_alignment);
                WriteCmuViInt32Value ("\t Lost Transport Blocks = ", lost_transp_blocks);
  
                printf ("\n\tMatching Limits?\n");
                printf ("\t-------------------\n");
  
                WriteCmuViInt32Value ("\t Bit Error Rate          : ", BER_lim_match);
                WriteCmuViInt32Value ("\t Block Error Rate        : ", BLER_lim_match);
                WriteCmuViInt32Value ("\t Data Block Error Rate   : ", DBLER_lim_match);
                
                
                
                ber_power = ber_power - step_size_ber; 
                } 
                while (DBLER_lim_match == 0);
                printf ("%s %f", "\n\n\tDBLER limit violated at output power: ", ber_power);
                dbler_pow_lim = ber_power+step_size_ber;
                }
                else
                {
                if (DBLER_lim_match != 0)
                  {
                  printf ("%s %f", "\n\n\tDBLER limit violated at output power: ", ber_power);
                  dbler_pow_lim = ber_power+step_size_ber;
                  do
                  {
                  CheckCmuError (RSCMU_WCDMA_Sig_ConfSigBSSigOutputPow (g_hRsCmuWcdma_Sig, ber_power, errorMessage));
                  CheckCmuError (RSCMU_WCDMA_NSig_ReadRXQuBER (g_hRsCmuWcdma_Sig, &ber, &bler, &dbler, &dl_ul_alignment,
                                                 &lost_transp_blocks, errorMessage));
                  CheckCmuError (RSCMU_WCDMA_NSig_CalcRCQuBERMatch (g_hRsCmuWcdma_Sig, &BER_lim_match, &BLER_lim_match,
                                                      &DBLER_lim_match, errorMessage));
                  
                  WriteCmuViInt32Value ("\n\n\t Measurement Status     = ", meas_stat_BER);
                  WriteCmuViInt32Value ("\t Counter stat. cycles   = ", counter_stat_cycle_BER);
                  WriteCmuViInt32Value ("\t Counter eval. periods   = ", counter_eval_period_BER);
                  WriteCmuViReal64Value ("\t Bit Error Rate        = ", ber);
                  WriteCmuViReal64Value ("\t Block Error Rate       = ", bler);
                  WriteCmuViReal64Value ("\t Data Block Error Rate  = ", dbler);
                  WriteCmuViInt32Value ("\t DL/UL Alignment         = ", dl_ul_alignment);
                  WriteCmuViInt32Value ("\t Lost Transport Blocks   = ", lost_transp_blocks);
  
                  printf ("\n\tMatching Limits?\n");
                  printf ("\t-------------------\n");
  
                  WriteCmuViInt32Value ("\t Bit Error Rate          : ", BER_lim_match);
                  WriteCmuViInt32Value ("\t Block Error Rate        : ", BLER_lim_match);
                  WriteCmuViInt32Value ("\t Data Block Error Rate   : ", DBLER_lim_match);
                  
                  
                  ber_power = ber_power - step_size_ber; 
                  } 
                  while (BLER_lim_match == 0);
                  printf ("%s %f", "\n\n\tBLER limit violated at output power: ", ber_power);
                  bler_pow_lim = ber_power+step_size_ber;
                  }
                }
              }
              else
              {
              //-------------------------------------------------------------------------
              if (BLER_lim_match != 0)
                {
                printf ("%s %f", "\n\n\tBLER limit violated at output power: ", ber_power); 
                bler_pow_lim = ber_power+step_size_ber;
                do
                {
                CheckCmuError (RSCMU_WCDMA_Sig_ConfSigBSSigOutputPow (g_hRsCmuWcdma_Sig, ber_power, errorMessage));
                CheckCmuError (RSCMU_WCDMA_NSig_ReadRXQuBER (g_hRsCmuWcdma_Sig, &ber, &bler, &dbler, &dl_ul_alignment,
                                                 &lost_transp_blocks, errorMessage));
                CheckCmuError (RSCMU_WCDMA_NSig_CalcRCQuBERMatch (g_hRsCmuWcdma_Sig, &BER_lim_match, &BLER_lim_match,
                                                      &DBLER_lim_match, errorMessage));
                ber_power1 = ber_power;
                ber1 = ber;
                bler1 = bler;
                dbler1 = dbler;
                dl_ul_alignment1 = dl_ul_alignment;
                lost_transport_blocks1 = lost_transp_blocks; 
                BER_lim_match1 = BER_lim_match;
                BLER_lim_match1 = BLER_lim_match;
                DBLER_lim_match1 = DBLER_lim_match;
                
                WriteCmuViInt32Value ("\n\n\t Measurement Status     = ", meas_stat_BER);
                WriteCmuViInt32Value ("\t Counter stat. cycles   = ", counter_stat_cycle_BER);
                WriteCmuViInt32Value ("\t Counter eval. periods     = ", counter_eval_period_BER);
                WriteCmuViReal64Value ("\t Bit Error Rate        = ", ber);
                WriteCmuViReal64Value ("\t Block Error Rate     = ", bler);
                WriteCmuViReal64Value ("\t Data Block Error Rate  = ", dbler);
                WriteCmuViInt32Value ("\t DL/UL Alignment       = ", dl_ul_alignment);
                WriteCmuViInt32Value ("\t Lost Transport Blocks = ", lost_transp_blocks);
  
                printf ("\n\tMatching Limits?\n");
                printf ("\t-------------------\n");
  
                WriteCmuViInt32Value ("\t Bit Error Rate          : ", BER_lim_match);
                WriteCmuViInt32Value ("\t Block Error Rate        : ", BLER_lim_match);
                WriteCmuViInt32Value ("\t Data Block Error Rate   : ", DBLER_lim_match);
                
                
                
                ber_power = ber_power - step_size_ber; 
                } 
                while (BER_lim_match == 0 && DBLER_lim_match == 0);
                if (BER_lim_match != 0)
                  {
                  printf ("%s %f", "\n\n\tBER limit violated at output power: ", ber_power);
                  ber_pow_lim = ber_power+step_size_ber;
                  do
                  {
                  CheckCmuError (RSCMU_WCDMA_Sig_ConfSigBSSigOutputPow (g_hRsCmuWcdma_Sig, ber_power, errorMessage));
                  CheckCmuError (RSCMU_WCDMA_NSig_ReadRXQuBER (g_hRsCmuWcdma_Sig, &ber, &bler, &dbler, &dl_ul_alignment,
                                                 &lost_transp_blocks, errorMessage));
                  CheckCmuError (RSCMU_WCDMA_NSig_CalcRCQuBERMatch (g_hRsCmuWcdma_Sig, &BER_lim_match, &BLER_lim_match,
                                                      &DBLER_lim_match, errorMessage));
                  ber_power2 = ber_power;
                  ber2 = ber;
                  bler2 = bler;
                  dbler2 = dbler;
                  dl_ul_alignment2 = dl_ul_alignment;
                  lost_transport_blocks2 = lost_transp_blocks; 
                  BER_lim_match2 = BER_lim_match;
                  BLER_lim_match2 = BLER_lim_match;
                  DBLER_lim_match2 = DBLER_lim_match;
                  
                  WriteCmuViInt32Value ("\n\n\t Measurement Status     = ", meas_stat_BER);
                  WriteCmuViInt32Value ("\t Counter stat. cycles   = ", counter_stat_cycle_BER);
                  WriteCmuViInt32Value ("\t Counter eval. periods   = ", counter_eval_period_BER);
                  WriteCmuViReal64Value ("\t Bit Error Rate        = ", ber);
                  WriteCmuViReal64Value ("\t Block Error Rate       = ", bler);
                  WriteCmuViReal64Value ("\t Data Block Error Rate  = ", dbler);
                  WriteCmuViInt32Value ("\t DL/UL Alignment         = ", dl_ul_alignment);
                  WriteCmuViInt32Value ("\t Lost Transport Blocks   = ", lost_transp_blocks);
  
                  printf ("\n\tMatching Limits?\n");
                  printf ("\t-------------------\n");
  
                  WriteCmuViInt32Value ("\t Bit Error Rate          : ", BER_lim_match);
                  WriteCmuViInt32Value ("\t Block Error Rate        : ", BLER_lim_match);
                  WriteCmuViInt32Value ("\t Data Block Error Rate   : ", DBLER_lim_match);
                  
                  
                  
                  ber_power = ber_power - step_size_ber; 
                  } 
                  while (DBLER_lim_match == 0);
                  printf ("%s %f", "\n\n\tDBLER limit violated at output power: ", ber_power);
                  dbler_pow_lim = ber_power+step_size_ber;
                  }
                  else
                  {
                  if (DBLER_lim_match != 0)
                    {
                    printf ("%s %f", "\n\n\tDBLER limit violated at output power: ", ber_power);
                    dbler_pow_lim = ber_power+step_size_ber;
                    do
                    {
                    CheckCmuError (RSCMU_WCDMA_Sig_ConfSigBSSigOutputPow (g_hRsCmuWcdma_Sig, ber_power, errorMessage));
                    CheckCmuError (RSCMU_WCDMA_NSig_ReadRXQuBER (g_hRsCmuWcdma_Sig, &ber, &bler, &dbler, &dl_ul_alignment,
                                                 &lost_transp_blocks, errorMessage));
                    CheckCmuError (RSCMU_WCDMA_NSig_CalcRCQuBERMatch (g_hRsCmuWcdma_Sig, &BER_lim_match, &BLER_lim_match,
                                                      &DBLER_lim_match, errorMessage));
                    
                    WriteCmuViInt32Value ("\n\n\t Measurement Status     = ", meas_stat_BER);
                    WriteCmuViInt32Value ("\t Counter stat. cycles   = ", counter_stat_cycle_BER);
                    WriteCmuViInt32Value ("\t Counter eval. periods     = ", counter_eval_period_BER);
                    WriteCmuViReal64Value ("\t Bit Error Rate        = ", ber);
                    WriteCmuViReal64Value ("\t Block Error Rate     = ", bler);
                    WriteCmuViReal64Value ("\t Data Block Error Rate  = ", dbler);
                    WriteCmuViInt32Value ("\t DL/UL Alignment       = ", dl_ul_alignment);
                    WriteCmuViInt32Value ("\t Lost Transport Blocks = ", lost_transp_blocks);
  
                    printf ("\n\tMatching Limits?\n");
                    printf ("\t-------------------\n");
  
                    WriteCmuViInt32Value ("\t Bit Error Rate          : ", BER_lim_match);
                    WriteCmuViInt32Value ("\t Block Error Rate        : ", BLER_lim_match);
                    WriteCmuViInt32Value ("\t Data Block Error Rate   : ", DBLER_lim_match);
                    
                    
                    
                    ber_power = ber_power - step_size_ber; 
                    } 
                    while (BER_lim_match == 0);
                    printf ("%s %f", "\n\n\tBER limit violated at output power: ", ber_power);
                    ber_pow_lim = ber_power+step_size_ber;
                    }
                  }
                }
                else
                {
                //----------------------------------------------------------------------------
                if (DBLER_lim_match != 0)
                  {
                  printf ("%s %f", "\n\n\tDBLER limit violated at output power: ", ber_power);
                  dbler_pow_lim = ber_power+step_size_ber;
                  do
                  {
                  CheckCmuError (RSCMU_WCDMA_Sig_ConfSigBSSigOutputPow (g_hRsCmuWcdma_Sig, ber_power, errorMessage));
                  CheckCmuError (RSCMU_WCDMA_NSig_ReadRXQuBER (g_hRsCmuWcdma_Sig, &ber, &bler, &dbler, &dl_ul_alignment,
                                                 &lost_transp_blocks, errorMessage));
                  CheckCmuError (RSCMU_WCDMA_NSig_CalcRCQuBERMatch (g_hRsCmuWcdma_Sig, &BER_lim_match, &BLER_lim_match,
                                                                                      &DBLER_lim_match, errorMessage));
                  ber_power1 = ber_power;
                  ber1 = ber;
                  bler1 = bler;
                  dbler1 = dbler;
                  dl_ul_alignment1 = dl_ul_alignment;
                  lost_transport_blocks1 = lost_transp_blocks;  
                  BER_lim_match1 = BER_lim_match;
                  BLER_lim_match1 = BLER_lim_match;
                  DBLER_lim_match1 = DBLER_lim_match;
                  
                  
                  WriteCmuViInt32Value ("\n\n\t Measurement Status     = ", meas_stat_BER);
                  WriteCmuViInt32Value ("\t Counter stat. cycles   = ", counter_stat_cycle_BER);
                  WriteCmuViInt32Value ("\t Counter eval. periods   = ", counter_eval_period_BER);
                  WriteCmuViReal64Value ("\t Bit Error Rate        = ", ber);
                  WriteCmuViReal64Value ("\t Block Error Rate       = ", bler);
                  WriteCmuViReal64Value ("\t Data Block Error Rate  = ", dbler);
                  WriteCmuViInt32Value ("\t DL/UL Alignment         = ", dl_ul_alignment);
                  WriteCmuViInt32Value ("\t Lost Transport Blocks   = ", lost_transp_blocks);
    
                  printf ("\n\tMatching Limits?\n");
                  printf ("\t-------------------\n");
    
                  WriteCmuViInt32Value ("\t Bit Error Rate          : ", BER_lim_match);
                  WriteCmuViInt32Value ("\t Block Error Rate        : ", BLER_lim_match);
                  WriteCmuViInt32Value ("\t Data Block Error Rate   : ", DBLER_lim_match);
                  
                  
                  
                  ber_power = ber_power - step_size_ber; 
                  } 
                  while (BER_lim_match == 0 && BLER_lim_match == 0);
                  if (BER_lim_match != 0)
                    {
                    printf ("%s %f", "\n\n\tBER limit violated at output power: ", ber_power);
                    ber_pow_lim = ber_power+step_size_ber;
                    do
                    {
                    CheckCmuError (RSCMU_WCDMA_Sig_ConfSigBSSigOutputPow (g_hRsCmuWcdma_Sig, ber_power, errorMessage));
                    CheckCmuError (RSCMU_WCDMA_NSig_ReadRXQuBER (g_hRsCmuWcdma_Sig, &ber, &bler, &dbler, &dl_ul_alignment,
                                                 &lost_transp_blocks, errorMessage));
                    CheckCmuError (RSCMU_WCDMA_NSig_CalcRCQuBERMatch (g_hRsCmuWcdma_Sig, &BER_lim_match, &BLER_lim_match,
                                                      &DBLER_lim_match, errorMessage));
                    ber_power2 = ber_power;
                    ber2 = ber;
                    bler2 = bler;
                    dbler2 = dbler;
                    dl_ul_alignment2 = dl_ul_alignment;
                    lost_transport_blocks2 = lost_transp_blocks;
                    BER_lim_match2 = BER_lim_match;
                    BLER_lim_match2 = BLER_lim_match;
                    DBLER_lim_match2 = DBLER_lim_match;
                    
                    WriteCmuViInt32Value ("\n\n\t Measurement Status     = ", meas_stat_BER);
                    WriteCmuViInt32Value ("\t Counter stat. cycles   = ", counter_stat_cycle_BER);
                    WriteCmuViInt32Value ("\t Counter eval. periods     = ", counter_eval_period_BER);
                    WriteCmuViReal64Value ("\t Bit Error Rate        = ", ber);
                    WriteCmuViReal64Value ("\t Block Error Rate     = ", bler);
                    WriteCmuViReal64Value ("\t Data Block Error Rate  = ", dbler);
                    WriteCmuViInt32Value ("\t DL/UL Alignment       = ", dl_ul_alignment);
                    WriteCmuViInt32Value ("\t Lost Transport Blocks = ", lost_transp_blocks);
    
                    printf ("\n\tMatching Limits?\n");
                    printf ("\t-------------------\n");
  
                    WriteCmuViInt32Value ("\t Bit Error Rate          : ", BER_lim_match);
                    WriteCmuViInt32Value ("\t Block Error Rate        : ", BLER_lim_match);
                    WriteCmuViInt32Value ("\t Data Block Error Rate   : ", DBLER_lim_match);
                    
                    
                    
                    ber_power = ber_power - step_size_ber; 
                    } 
                    while (BLER_lim_match == 0);
                    printf ("%s %f", "\n\n\tBLER limit violated at output power: ", ber_power);
                    bler_pow_lim = ber_power+step_size_ber;
                    }
                    else
                    {
                    if (BLER_lim_match != 0)
                      {
                      printf ("%s %f", "\n\n\tBLER limit violated at output power: ", ber_power);
                      bler_pow_lim = ber_power+step_size_ber;
                      do
                      {
                      CheckCmuError (RSCMU_WCDMA_Sig_ConfSigBSSigOutputPow (g_hRsCmuWcdma_Sig, ber_power, errorMessage));
                      CheckCmuError (RSCMU_WCDMA_NSig_ReadRXQuBER (g_hRsCmuWcdma_Sig, &ber, &bler, &dbler, &dl_ul_alignment,
                                                 &lost_transp_blocks, errorMessage));
                      CheckCmuError (RSCMU_WCDMA_NSig_CalcRCQuBERMatch (g_hRsCmuWcdma_Sig, &BER_lim_match, &BLER_lim_match,
                                                      &DBLER_lim_match, errorMessage));
                      
                      WriteCmuViInt32Value ("\n\n\t Measurement Status     = ", meas_stat_BER);
                      WriteCmuViInt32Value ("\t Counter stat. cycles   = ", counter_stat_cycle_BER);
                      WriteCmuViInt32Value ("\t Counter eval. periods   = ", counter_eval_period_BER);
                      WriteCmuViReal64Value ("\t Bit Error Rate        = ", ber);
                      WriteCmuViReal64Value ("\t Block Error Rate       = ", bler);
                      WriteCmuViReal64Value ("\t Data Block Error Rate  = ", dbler);
                      WriteCmuViInt32Value ("\t DL/UL Alignment         = ", dl_ul_alignment);
                      WriteCmuViInt32Value ("\t Lost Transport Blocks   = ", lost_transp_blocks);
  
                      printf ("\n\tMatching Limits?\n");
                      printf ("\t-------------------\n");
  
                      WriteCmuViInt32Value ("\t Bit Error Rate          : ", BER_lim_match);
                      WriteCmuViInt32Value ("\t Block Error Rate        : ", BLER_lim_match);
                      WriteCmuViInt32Value ("\t Data Block Error Rate   : ", DBLER_lim_match);
                      
                      
                      ber_power = ber_power - step_size_ber; 
                      } 
                      while (BER_lim_match == 0);
                      printf ("%s %f", "\n\n\tBER limit violated at output power: ", ber_power);
                      ber_pow_lim = ber_power+step_size_ber;
                      }
                    }
                  }
                  else {}
                }  
              }
            }
          }
        }
      }
    
    
    CheckCmuError (RSCMU_WCDMA_NSig_AborRXQuBER (g_hRsCmuWcdma_Sig, errorMessage));
  
  
  printf ("\n\n\n\tReceiver Quality Measurement\n");
  printf ("\t--------------------------------\n");
  
  WriteCmuViInt32Value ("\t Measurement Status     = ", meas_stat_BER);
  WriteCmuViInt32Value ("\t Counter stat. cycles   = ", counter_stat_cycle_BER);
  WriteCmuViInt32Value ("\t Counter eval. periods   = ", counter_eval_period_BER);
  WriteCmuViReal64Value ("\t Bit Error Rate        = ", ber);
  WriteCmuViReal64Value ("\t Block Error Rate       = ", bler);
  WriteCmuViReal64Value ("\t Data Block Error Rate  = ", dbler);
  WriteCmuViInt32Value ("\t DL/UL Alignment         = ", dl_ul_alignment);
  WriteCmuViInt32Value ("\t Lost Transport Blocks   = ", lost_transp_blocks);
  
  printf ("\n\tMatching Limits?\n");
  printf ("\t-------------------\n");
  
  WriteCmuViInt32Value ("\t Bit Error Rate          : ", BER_lim_match);
  WriteCmuViInt32Value ("\t Block Error Rate        : ", BLER_lim_match);
  WriteCmuViInt32Value ("\t Data Block Error Rate   : ", DBLER_lim_match);
  };
  fflush (stdin);
  
  
  //-----------------------------------------------------------
  // Save the results to a .txt file in the current directory
  //-----------------------------------------------------------
  
  printf ("%s", "\n\n\nSave the results to Results_Sig.txt? (y/n)");
 
  scanf("%c",&a);
  fflush(stdin);
  if (a == 'y')
    {
    fp = fopen("Results_Sig.txt", "w");
  
    //--------------------------
    // Settings
    //--------------------------
    
    fprintf (fp, "%s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s", 
    "Settings:\t(1 = yes, 0 = no)\n\nSignalling State: \t\t\t\t\t\t", signalling_state, 
    "\nRegistration Identity Type: \t\t\t\t\t", registration_identity_type, 
    "\nSupport of Lossless SRNS Relocation? \t\t\t\t",lossles_srns_relocation, 
    "\nSupport of RFC 2507? \t\t\t\t\t\t", rf_2507, 
    "\nHeader Compression = \t\t\t\t\t\t", header_compression, 
    "\nAM Buffer Size = \t\t\t\t\t\t", am_buffer_size,
    "\nMaximum RLC Window Size = \t\t\t\t\t", max_rlc_window_size, 
    "\nMaximum AM Entities = \t\t\t\t\t\t", max_am_entities, 
    "\nSimult. Transport Channels (Downlink): \t\t\t\t", sim_transport_ch_d, 
    "\nSimult. CCTr. Channel (Downlink): \t\t\t\t", sim_cctrch_d, 
    "\nTTI Transport Blocks (Downlink): \t\t\t\t", tti_trans_blocks_d, 
    "\nNumber Of TFC (Downlink): \t\t\t\t\t", number_of_tfc_d, 
    "\nNumber Of TF (Downlink): \t\t\t\t\t", number_of_tf_d, 
    "\nTurbo Decoding (Downlink)? \t\t\t\t\t", turbo_decoding_d, 
    "\nReceived Bits All (Downlink): \t\t\t\t\t", rec_bits_all_d, 
    "\nReceived Bits Conv. Coded (Downlink): \t\t\t\t", rec_bits_conv_coded_d, 
    "\nReceived Bits Turbo Coded (Downlink) = \t\t\t\t", rec_bits_turbo_coded_d, 
    "\nDPCH/PDSCH Codes (Downlink): \t\t\t\t\t", dpch_codes_d,
    "\nPhysical Channel Bits (Downlink): \t\t\t\t", physical_chn_bits_d, 
    "\nSupport of SF 512? \t\t\t\t\t\t", sf_512_d, 
    /*"\nSupport of PDSCH? \t\t\t\t\t\t", pdsch_d, */
    /*"\nSupport of Channel Estimation Pilots? \t\t\t\t", chn_est_pilots_d, */
    /*"\nSimultaneous Reception of SCCPCH snd DPCH? \t\t\t", sccpch_dpch_d,  */
    /*"\nSimultaneous Reception of SCCPCH, DPCH and PDSCH? \t\t", sccpch_dpch_pdsch_d,  */
    /*"\nSimultaneous Reception of SCCPCH radio links? \t\t\t", sccpch_radio_links_d, */
    "\nSimult. Transport Channels (Uplink): \t\t\t\t", sim_transport_ch_u, 
    "\nSimult. CCTr. Channel (Uplink): \t\t\t\t", sim_cctrch_u,
    "\nTTI Transport Blocks (Uplink): \t\t\t\t\t", tti_trans_blocks_u, 
    "\nNumber Of TFC (Uplink): \t\t\t\t\t", number_of_tfc_u, 
    "\nNumber Of TF (Uplink): \t\t\t\t\t\t", number_of_tf_u, 
    "\nTurbo Decoding (Uplink)? \t\t\t\t\t", turbo_decoding_u, 
    "\nReceived Bits All (Uplink): \t\t\t\t\t", rec_bits_all_u, 
    "\nReceived Bits Conv. Coded (Uplink): \t\t\t\t", rec_bits_conv_coded_u, 
    "\nReceived Bits Turbo Coded (Uplink): \t\t\t\t", rec_bits_turbo_coded_u, 
    "\nDPDCH Bits per 10 ms: \t\t\t\t\t\t", dpdch_bits_per_10ms, 
    "\nPhysical Channel FDD PCPCH? \t\t\t\t\t", pcpch, 
    "\nUE Power Class: \t\t\t\t\t\t", ue_power_class, 
    "\nTx/Rx Frequency Separation: \t\t\t\t\t", freq_separation,
    "\nUTRA FDD/TDD? \t\t\t\t\t\t\t", utra_fdd_tdd, 
    "\nGSM Support? \t\t\t\t\t\t\t", gsm, 
    "\nMulti-Carrier Mode Support?\t\t\t\t\t", multi_carr_mode, 
    "\nSupport of DL Compressed Mode? \t\t\t\t\t", dl_compressed_mode, 
    "\nSupport of UL Compressed Mode? \t\t\t\t\t", ul_compressed_mode, 
    "\n\n\n");
    
    //--------------------------
    // Max. Power
    //--------------------------
    
    fprintf(fp, "%s %f %s %f %s %f %s %f %s %f %s %f %s %f %s %f %s", 
    "Maximum Power Measurement:\n\nCurrent Peak Power: \t" ,  Peak_Power_Current, 
    " dBm\nAverage Peak Power: \t", Peak_Power_Average, 
    " dBm\nMaximum Peak Power: \t", Peak_Power_Maximum, 
    " dBm\nCurrent RMS Power: \t", RMS_Power_Current, 
    " dBm\nAverage RMS Power: \t", RMS_Power_Average, 
    " dBm\nMaximum RMS Power: \t",RMS_Power_Maximum, 
    " dBm\nMinimum RMS Power: \t", RMS_Power_Minimum, 
    " dBm\nOut Of Tolerance: \t", Out_Of_Tolerance, 
    " %\n\n\n");              
                  
    //--------------------------
    // WCDMA Overview
    //--------------------------
  
    fprintf(fp, "%s %f %s %f %s %f %s %f %s %f %s %f %s %f %s %f %s %f %s %f %s %f %s %f %s %f %s %f %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s %i %s",
    "WCDMA Modulation Overview Measurement:\n\nEVM Peak: \t\t\t", evm_peak_wcdma, 
    " %\nEVM RMS: \t\t\t", evm_rms_wcdma, 
    " %\nMagnitude Error Peak: \t\t", magn_error_wcdma, 
    " %\nMagnitude Error RMS: \t\t", magn_error_rms_wcdma, 
    " %\nPhase Error Peak: \t\t", phase_error_peak_wcdma, 
    " deg\nPhase Error RMS: \t\t", phase_error_rms_wcdma, 
    " deg\nI/Q Origin Offset: \t\t", iq_origin_offset_wcdma, 
    " dB\nI/Q Imbalance: \t\t\t", iq_imbalance_wcdma, 
    " dB\nFrequency Error: \t\t", freq_err_wcdma, 
    " Hz\nWaveform Quality: \t\t", wav_quality_wcdma, 
    "\nPeak Code Domain Error: \t", peak_cd_err_wcdma, 
    " dB\nTransmit Time Error: \t\t", transmit_time_err_wcdma, 
    " s\nCurrent UE Power: \t\t", ue_pow_curr_wcdma, 
    " dBm\nOut Of Tolerance: \t\t", out_of_tolerance_wcdma, 
    " %\nSlot Number: \t\t\t", slot_nr_wcdma, 
    "\n\nMatching Limits?\t(0 = okay, 1 = invalid, 2 = underflow, 3 = overflow)\n\nEVM Peak: \t\t\t", error_vector_magn_peak_wcdma_matc, 
    "\nEVM RMS: \t\t\t", error_vector_magn_rms_wcdma_matc, 
    "\nMagnitude Error Peak: \t\t", magn_err_peak_wcdma_matc, 
    "\nMagnitude Error RMS: \t\t", magn_err_rms_wcdma_matc, 
    "\nPhase Error Peak: \t\t", phase_err_peak_wcdma_matc,
    "\nPhase Error RMS: \t\t", phase_err_rms_wcdma_matc, 
    "\nI/Q Origin Offset: \t\t", iq_origin_offset_wcdma_matc, 
    "\nI/Q Imbalance: \t\t\t", iq_imbalance_wcdma_matc, 
    "\nFrequency Error: \t\t", freq_err_wcdma_matc, 
    "\nWaveform Quality: \t\t", wav_qual_wcdma_matc, 
    "\nPeak Code Domain Error: \t", peak_cd_err_wcdma_matc,
    "\nTransmit Time Error: \t\t", transmit_time_err_wcdma_matc, 
    "\n\n\n");              
                  
                  
    //--------------------------
    // Receiver Quality
    //--------------------------
                 
    fprintf(fp, "%s %f %s %f %s %f ", 
    "\n\n\nBER limit violated at output power: \t", ber_pow_lim,
    " dB\nBLER limit violated at output power: \t", bler_pow_lim,
    " dB\nDBLER limit violated at output power: \t", dbler_pow_lim);
    
    
    if (ber2 != 0)
      {
      fprintf (fp, "%s %f %s %i %s %i %s %f %s %f %s %f %s %i %s %i %s %i %s %i %s %i %s",
      " dB\n\nValues at ", ber_power2+step_size_ber, 
      " dB: \n\nCounter stat. cycles: \t", counter_stat_cycle_BER, 
      "\nCounter eval. periods: \t", counter_eval_period_BER, 
      "\nBit Error Rate  : \t", ber2, 
      " %\nBlock Error Rate: \t", bler2, 
      " %\nData Block Error Rate: \t",dbler2, 
      " %\nDL/UL Alignment: \t", dl_ul_alignment2, 
      "\nLost Transport Blocks: \t", lost_transport_blocks2,
      "\n\nMatching Limits?\t(0 = unknown, 1 = invalid, 2 = okay, 3 = not matching limits)\n\nBit Error Rate: \t", BER_lim_match2, 
      "\nBlock Error Rate: \t", BLER_lim_match2, 
      "\nData Block Error Rate: \t", DBLER_lim_match2, 
      "\n\n\n");              
      };
    
    if (ber1 != 0)
      {
      fprintf (fp, "%s %f %s %i %s %i %s %f %s %f %s %f %s %i %s %i %s %i %s %i %s %i %s",
      " dB\n\nValues at ", ber_power1+step_size_ber, 
      " dB: \n\nCounter stat. cycles: \t", counter_stat_cycle_BER, 
      "\nCounter eval. periods: \t", counter_eval_period_BER, 
      "\nBit Error Rate  : \t", ber1, 
      " %\nBlock Error Rate: \t", bler1, 
      " %\nData Block Error Rate: \t",dbler1, 
      " %\nDL/UL Alignment: \t", dl_ul_alignment1, 
      "\nLost Transport Blocks: \t", lost_transport_blocks1,
      "\n\nMatching Limits?\t(0 = unknown, 1 = invalid, 2 = okay, 3 = not matching limits)\n\nBit Error Rate: \t", BER_lim_match1, 
      "\nBlock Error Rate: \t", BLER_lim_match1, 
      "\nData Block Error Rate: \t", DBLER_lim_match1, 
      "\n\n\n");              
      };
    
    
     fprintf (fp, "%s %f %s %i %s %i %s %f %s %f %s %f %s %i %s %i %s %i %s %i %s %i %s",
      " dB\n\nValues at ", ber_power+step_size_ber, 
      " dB: \n\nCounter stat. cycles: \t", counter_stat_cycle_BER, 
      "\nCounter eval. periods: \t", counter_eval_period_BER, 
      "\nBit Error Rate  : \t", ber, 
      " %\nBlock Error Rate: \t", bler, 
      " %\nData Block Error Rate: \t",dbler, 
      " %\nDL/UL Alignment: \t", dl_ul_alignment, 
      "\nLost Transport Blocks: \t", lost_transp_blocks,
      "\n\nMatching Limits?\t(0 = unknown, 1 = invalid, 2 = okay, 3 = not matching limits)\n\nBit Error Rate: \t", BER_lim_match, 
      "\nBlock Error Rate: \t", BLER_lim_match, 
      "\nData Block Error Rate: \t", DBLER_lim_match, 
      "\n\n\n");              
      
    fclose(fp); 
    };
     
  
  
  
  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // DISCONNECT THE UE
  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  
   printf ("%s", "\n\nDisconnect the UE? (y/n)");
  
   scanf("%c",&d);
  
   if (d == 'y')
    {
    CheckCmuError   (RSCMU_WCDMA_Sig_ProcSignAct (g_hRsCmuWcdma_Sig, RSCMU_WCDMA_CREL, errorMessage));
    CheckCmuError (RSCMU_WCDMA_Sig_SensSignStat (g_hRsCmuWcdma_Sig, &signalling_state, errorMessage));
    
       if (signalling_state == 2)
          {
          printf ("\n\t UE Disconnected\n");
          }
       else
          {
          printf ("\n\t Disconnection failed\n");
          };
   };   
   fflush (stdin);
   
  
   
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   // UNREGISTER THE UE
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   
   printf ("%s", "\n\nUnregsiter the UE? (y/n)");
   
   scanf ("%c", &e);
   
   if (e == 'y')
    {
     CheckCmuError  (RSCMU_WCDMA_Sig_ProcSignAct (g_hRsCmuWcdma_Sig, RSCMU_WCDMA_UREG, errorMessage));
     CheckCmuError (RSCMU_WCDMA_Sig_SensSignStat (g_hRsCmuWcdma_Sig, &signalling_state, errorMessage));
  
        if (signalling_state == 1)
           {
           printf ("\n\t UE Unregistered\n");
           }
        else
           {
           printf ("\n\t Unregistration failed\n");
           };
    };
  fflush (stdin);
    
    
  
  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // SWITCH OFF SIGNAL
  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
  printf ("%s", "\n\nSwitch off Signal? (y/n)");
    
  scanf ("%c", &f);
    
  if (f == 'y')
   {
    CheckCmuError   (RSCMU_WCDMA_Sig_ProcSignAct (g_hRsCmuWcdma_Sig, RSCMU_WCDMA_SOFF, errorMessage));
    CheckCmuError (RSCMU_WCDMA_Sig_SensSignStat (g_hRsCmuWcdma_Sig, &signalling_state, errorMessage));
     
        if (signalling_state == 0)
           {
           printf ("\n\t Signal Switched Off\n");
           }
        else
           {
           printf ("\n\t Switch Off failed\n");
           };
  };
  fflush (stdin);
     
 
 
  
 
  
 
 
  // Close the file "RsCmu.iee".
 // CheckCmuError (RSCMU_Trace_OFF (g_hRsCmuBase, errorMessage));
  
  CheckCmuError (RSCMU_Close_Base (g_hRsCmuBase, errorMessage));    

  return (0);

  Error:
 return (-1);
 }

 
 
 
 
 
 
 
 
 
 
